package com.qxgmat.data.dao.entity;

import java.io.Serializable;
import javax.persistence.*;

@Table(name = "user_paper")
public class UserPaper implements Serializable {
    @Id
    @Column(name = "`id`")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    /**
     * 用户id
     */
    @Column(name = "`user_id`")
    private Integer userId;

    /**
     * 组卷名称
     */
    @Column(name = "`title`")
    private String title;

    /**
     * 组卷模块：examination，exercise，collect，error，homework_preview
     */
    @Column(name = "`module`")
    private String module;

    /**
     * 对应模块id
     */
    @Column(name = "`module_id`")
    private Integer moduleId;

    /**
     * 题目id列表：json
     */
    @Column(name = "`question_ids`")
    private Integer[] questionIds;

    /**
     * 练习次数
     */
    @Column(name = "`number`")
    private Integer number;

    /**
     * 总时间
     */
    @Column(name = "`time`")
    private Integer time;

    private static final long serialVersionUID = 1L;

    /**
     * @return id
     */
    public Integer getId() {
        return id;
    }

    /**
     * @param id
     */
    public void setId(Integer id) {
        this.id = id;
    }

    /**
     * 获取用户id
     *
     * @return user_id - 用户id
     */
    public Integer getUserId() {
        return userId;
    }

    /**
     * 设置用户id
     *
     * @param userId 用户id
     */
    public void setUserId(Integer userId) {
        this.userId = userId;
    }

    /**
     * 获取组卷名称
     *
     * @return title - 组卷名称
     */
    public String getTitle() {
        return title;
    }

    /**
     * 设置组卷名称
     *
     * @param title 组卷名称
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * 获取组卷模块：examination，exercise，collect，error，homework_preview
     *
     * @return module - 组卷模块：examination，exercise，collect，error，homework_preview
     */
    public String getModule() {
        return module;
    }

    /**
     * 设置组卷模块：examination，exercise，collect，error，homework_preview
     *
     * @param module 组卷模块：examination，exercise，collect，error，homework_preview
     */
    public void setModule(String module) {
        this.module = module;
    }

    /**
     * 获取对应模块id
     *
     * @return module_id - 对应模块id
     */
    public Integer getModuleId() {
        return moduleId;
    }

    /**
     * 设置对应模块id
     *
     * @param moduleId 对应模块id
     */
    public void setModuleId(Integer moduleId) {
        this.moduleId = moduleId;
    }

    /**
     * 获取题目id列表：json
     *
     * @return question_ids - 题目id列表：json
     */
    public Integer[] getQuestionIds() {
        return questionIds;
    }

    /**
     * 设置题目id列表：json
     *
     * @param questionIds 题目id列表：json
     */
    public void setQuestionIds(Integer[] questionIds) {
        this.questionIds = questionIds;
    }

    /**
     * 获取练习次数
     *
     * @return number - 练习次数
     */
    public Integer getNumber() {
        return number;
    }

    /**
     * 设置练习次数
     *
     * @param number 练习次数
     */
    public void setNumber(Integer number) {
        this.number = number;
    }

    /**
     * 获取总时间
     *
     * @return time - 总时间
     */
    public Integer getTime() {
        return time;
    }

    /**
     * 设置总时间
     *
     * @param time 总时间
     */
    public void setTime(Integer time) {
        this.time = time;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getClass().getSimpleName());
        sb.append(" [");
        sb.append("Hash = ").append(hashCode());
        sb.append(", id=").append(id);
        sb.append(", userId=").append(userId);
        sb.append(", title=").append(title);
        sb.append(", module=").append(module);
        sb.append(", moduleId=").append(moduleId);
        sb.append(", questionIds=").append(questionIds);
        sb.append(", number=").append(number);
        sb.append(", time=").append(time);
        sb.append("]");
        return sb.toString();
    }

    public static UserPaper.Builder builder() {
        return new UserPaper.Builder();
    }

    public static class Builder {
        private UserPaper obj;

        public Builder() {
            this.obj = new UserPaper();
        }

        /**
         * @param id
         */
        public Builder id(Integer id) {
            obj.setId(id);
            return this;
        }

        /**
         * 设置用户id
         *
         * @param userId 用户id
         */
        public Builder userId(Integer userId) {
            obj.setUserId(userId);
            return this;
        }

        /**
         * 设置组卷名称
         *
         * @param title 组卷名称
         */
        public Builder title(String title) {
            obj.setTitle(title);
            return this;
        }

        /**
         * 设置组卷模块：examination，exercise，collect，error，homework_preview
         *
         * @param module 组卷模块：examination，exercise，collect，error，homework_preview
         */
        public Builder module(String module) {
            obj.setModule(module);
            return this;
        }

        /**
         * 设置对应模块id
         *
         * @param moduleId 对应模块id
         */
        public Builder moduleId(Integer moduleId) {
            obj.setModuleId(moduleId);
            return this;
        }

        /**
         * 设置题目id列表：json
         *
         * @param questionIds 题目id列表：json
         */
        public Builder questionIds(Integer[] questionIds) {
            obj.setQuestionIds(questionIds);
            return this;
        }

        /**
         * 设置练习次数
         *
         * @param number 练习次数
         */
        public Builder number(Integer number) {
            obj.setNumber(number);
            return this;
        }

        /**
         * 设置总时间
         *
         * @param time 总时间
         */
        public Builder time(Integer time) {
            obj.setTime(time);
            return this;
        }

        public UserPaper build() {
            return this.obj;
        }
    }
}