package com.qxgmat.controller.admin;

import com.alibaba.fastjson.JSONObject;
import com.nuliji.tools.Response;
import com.nuliji.tools.ResponseHelp;
import com.qxgmat.data.constants.enums.SettingKey;
import com.qxgmat.data.dao.entity.Setting;
import com.qxgmat.service.inline.SettingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;


@RestController("AdminSettingController")
@RequestMapping("/admin/setting")
@Api(tags = "配置信息接口", description = "全局独立配置设置", produces = MediaType.APPLICATION_JSON_VALUE)
public class SettingController {
    private static final Logger logger = LoggerFactory.getLogger(SettingController.class);

    @Autowired
    private SettingService settingService;

    @RequestMapping(value = "/index", method = RequestMethod.PUT)
    @ApiOperation(value = "修改首页配置", httpMethod = "PUT")
    private Response<Boolean> indexEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.INDEX.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/index", method = RequestMethod.GET)
    @ApiOperation(value = "获取首页配置", httpMethod = "GET")
    private Response<JSONObject> indexGet(){
        Setting entity = settingService.getByKey(SettingKey.INDEX.key);
        logger.debug("{}", entity);
        return ResponseHelp.success(entity.getValue());
    }

    @RequestMapping(value = "/place", method = RequestMethod.PUT)
    @ApiOperation(value = "修改考点设置", httpMethod = "PUT")
    private Response<Boolean> placeEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.PLACE.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/place", method = RequestMethod.GET)
    @ApiOperation(value = "获取考点配置", httpMethod = "GET")
    private Response<JSONObject> placeGet(){
        Setting entity = settingService.getByKey(SettingKey.PLACE.key);

        return ResponseHelp.success(entity.getValue());
    }


    @RequestMapping(value = "/sentence", method = RequestMethod.PUT)
    @ApiOperation(value = "修改长难句设置", httpMethod = "PUT")
    private Response<Boolean> sentenceEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.SENTENCE.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/sentence", method = RequestMethod.GET)
    @ApiOperation(value = "获取长难句配置", httpMethod = "GET")
    private Response<JSONObject> sentenceGet(){
        Setting entity = settingService.getByKey(SettingKey.SENTENCE.key);

        return ResponseHelp.success(entity.getValue());
    }


    @RequestMapping(value = "/exercise_time", method = RequestMethod.PUT)
    @ApiOperation(value = "修改做题时间设置", httpMethod = "PUT")
    private Response<Boolean> exerciseTimeEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.EXERCISE_TIME.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/exercise_time", method = RequestMethod.GET)
    @ApiOperation(value = "获取做题时间配置", httpMethod = "GET")
    private Response<JSONObject> exerciseTimeGet(){
        Setting entity = settingService.getByKey(SettingKey.EXERCISE_TIME.key);

        return ResponseHelp.success(entity.getValue());
    }
    @RequestMapping(value = "/examination_time", method = RequestMethod.PUT)
    @ApiOperation(value = "修改做题时间设置", httpMethod = "PUT")
    private Response<Boolean> examinationTimeEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.EXAMINATION_TIME.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/examination_time", method = RequestMethod.GET)
    @ApiOperation(value = "获取做题时间配置", httpMethod = "GET")
    private Response<JSONObject> examinationTimeGet(){
        Setting entity = settingService.getByKey(SettingKey.EXAMINATION_TIME.key);

        return ResponseHelp.success(entity.getValue());
    }
    @RequestMapping(value = "/filter_time", method = RequestMethod.PUT)
    @ApiOperation(value = "修改剔除时间设置", httpMethod = "PUT")
    private Response<Boolean> filterTimeEdit(@RequestBody @Validated JSONObject dto){
        Setting entity = settingService.getByKey(SettingKey.FILTER_TIME.key);
        entity.setValue(dto);
        settingService.edit(entity);
        return ResponseHelp.success(true);
    }

    @RequestMapping(value = "/filter_time", method = RequestMethod.GET)
    @ApiOperation(value = "获取做题时间配置", httpMethod = "GET")
    private Response<JSONObject> filterTimeGet(){
        Setting entity = settingService.getByKey(SettingKey.FILTER_TIME.key);

        return ResponseHelp.success(entity.getValue());
    }
}
