package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.qxgmat.data.dao.ExercisePaperMapper;
import com.qxgmat.data.dao.entity.ExercisePaper;
import com.qxgmat.data.dao.entity.Question;
import com.qxgmat.service.inline.ExercisePaperQuestionService;
import com.qxgmat.service.inline.QuestionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class ExercisePaperService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(ExercisePaperService.class);

    @Resource
    private ExercisePaperMapper exercisePaperMapper;

    @Resource
    private ExercisePaperQuestionService exercisePaperQuestionService;

    @Resource
    private QuestionService questionService;

    @Transactional
    public Question addQuestion(Question question){
        question = questionService.add(question);
        // 进行分组逻辑处理
//        SentenceQuestion entity = Transform.convert(relation, SentenceQuestion.class);
//        entity.setQuestionId(question.getId());
//        return add(entity);
        return question;
    }

    @Transactional
    public Question editQuestion(Question question){
        question = questionService.edit(question);
//        SentenceQuestion entity = Transform.convert(relation, SentenceQuestion.class);
//        return edit(entity);
        return question;
    }
    public ExercisePaper add(ExercisePaper paper){
        int result = insert(exercisePaperMapper, paper);
        paper = one(exercisePaperMapper, paper.getId());
        if(paper == null){
            throw new SystemException("组卷添加失败");
        }
        return paper;
    }

    public ExercisePaper edit(ExercisePaper paper){
        ExercisePaper in = one(exercisePaperMapper, paper.getId());
        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        int result = update(exercisePaperMapper, paper);
        return paper;
    }

    public boolean delete(Number id){
        ExercisePaper in = one(exercisePaperMapper, id);
        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        int result = delete(exercisePaperMapper, id);
        return result > 0;
    }

    public ExercisePaper selectOne(Number id){
        ExercisePaper in = one(exercisePaperMapper, id);

        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        return in;
    }

    public Page<ExercisePaper> select(int page, int pageSize){
        return select(exercisePaperMapper, page, pageSize);
    }

    public List<ExercisePaper> select(Collection ids){
        return select(exercisePaperMapper, ids);
    }

}
