package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.qxgmat.data.dao.SentencePaperMapper;
import com.qxgmat.data.dao.entity.Question;
import com.qxgmat.data.dao.entity.SentencePaper;
import com.qxgmat.data.dao.entity.SentenceQuestion;
import com.qxgmat.data.relation.entity.SentenceQuestionRelation;
import com.qxgmat.service.inline.QuestionService;
import com.qxgmat.service.inline.SentenceQuestionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class SentencePaperService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(SentencePaperService.class);

    @Resource
    private SentencePaperMapper sentencePaperMapper;

    @Resource
    private QuestionService questionService;

    @Resource
    private SentenceQuestionService sentenceQuestionService;

    @Transactional
    public SentenceQuestion addQuestion(SentenceQuestionRelation relation){
        Question question = relation.getQuestion();
        question = questionService.add(question);
        // 根据序号创建
        SentenceQuestion entity = Transform.convert(relation, SentenceQuestion.class);
        entity.setQuestionId(question.getId());
        return sentenceQuestionService.add(entity);
    }

    @Transactional
    public SentenceQuestion editQuestion(SentenceQuestionRelation relation){
        Question question = relation.getQuestion();
        question = questionService.edit(question);
        // 根据序号调整分组
        SentenceQuestion entity = Transform.convert(relation, SentenceQuestion.class);
        return sentenceQuestionService.edit(entity);
    }

    public SentencePaper add(SentencePaper paper){
        int result = insert(sentencePaperMapper, paper);
        paper = one(sentencePaperMapper, paper.getId());
        if(paper == null){
            throw new SystemException("题目添加失败");
        }
        return paper;
    }

    public SentencePaper edit(SentencePaper paper){
        SentencePaper in = one(sentencePaperMapper, paper.getId());
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = update(sentencePaperMapper, paper);
        return paper;
    }

    public boolean delete(Number id){
        SentencePaper in = one(sentencePaperMapper, id);
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = delete(sentencePaperMapper, id);
        return result > 0;
    }

    public SentencePaper selectOne(Number id){
        SentencePaper in = one(sentencePaperMapper, id);

        if(in == null){
            throw new ParameterException("题目不存在");
        }
        return in;
    }

    public Page<SentencePaper> select(int page, int pageSize){
        return select(sentencePaperMapper, page, pageSize);
    }

    public List<SentencePaper> select(Collection ids){
        return select(sentencePaperMapper, ids);
    }

}
