package com.qxgmat.service.inline;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.qxgmat.data.constants.enums.status.HomeworkPreviewStatus;
import com.qxgmat.data.dao.HomeworkPreviewMapper;
import com.qxgmat.data.dao.entity.HomeworkPreview;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;

@Service
public class HomeworkPreviewService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(HomeworkPreviewService.class);

    @Resource
    private HomeworkPreviewMapper homeworkPreviewMapper;

    public HomeworkPreview add(HomeworkPreview homeworkPreview){
        int result = insert(homeworkPreviewMapper, homeworkPreview);
        homeworkPreview = one(homeworkPreviewMapper, homeworkPreview.getId());
        if(homeworkPreview == null){
            throw new SystemException("预习作业添加失败");
        }
        return homeworkPreview;
    }

    public HomeworkPreview edit(HomeworkPreview homeworkPreview){
        HomeworkPreview in = one(homeworkPreviewMapper, homeworkPreview.getId());
        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        int result = update(homeworkPreviewMapper, homeworkPreview);
        return homeworkPreview;
    }

    public boolean delete(Number id){
        HomeworkPreview in = one(homeworkPreviewMapper, id);
        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        int result = delete(homeworkPreviewMapper, id);
        return result > 0;
    }

    public HomeworkPreview selectOne(Number id){
        HomeworkPreview in = one(homeworkPreviewMapper, id);

        if(in == null){
            throw new ParameterException("预习作业不存在");
        }
        return in;
    }

    public Page<HomeworkPreview> select(int page, int pageSize){
        return select(homeworkPreviewMapper, page, pageSize);
    }

    public Page<HomeworkPreview> select(int page, int pageSize, Integer category, HomeworkPreviewStatus status){
        Example example = new Example(HomeworkPreview.class);
        if(category != null && category>0)
            example.and(
                    example.createCriteria().andEqualTo("category", category)
            );
        if (status!=HomeworkPreviewStatus.ALL){
            tk.mybatis.mapper.entity.Example.Criteria criteria = example.createCriteria();
            switch(status){
                case NO_START:
                    criteria.andLessThan("startTime", new Date());
                    break;
                case PROCESS:
                    criteria.andGreaterThanOrEqualTo("startTime", new Date()).andLessThan("endTime", new Date());
                    break;
                case FINISH:
                    criteria.andGreaterThanOrEqualTo("endTime", new Date());
                    break;
            }
            example.and(criteria);
        }
        example.orderBy("id").desc();
        return select(homeworkPreviewMapper, example, page, pageSize);
    }

    public List<HomeworkPreview> select(Collection ids){
        return select(homeworkPreviewMapper, ids);
    }

}
