package com.qxgmat.service.inline;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.dao.HomeworkPreviewMapper;
import com.qxgmat.data.dao.QuestionMapper;
import com.qxgmat.data.dao.entity.HomeworkPreview;
import com.qxgmat.data.dao.entity.Question;
import com.qxgmat.data.relation.QuestionRelationMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class QuestionService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(QuestionService.class);

    @Resource
    private QuestionMapper questionMapper;

    @Resource
    private QuestionRelationMapper questionRelationMapper;

    public Page<Question> listExerciseAdmin(int page, int pageSize, Number structId, Number questionNo, Number paperId, String place, String difficult, String startTime, String endTime, String order, DirectionStatus direction){

        if(order.isEmpty()) order = "id";
        if (direction == null){
            direction = DirectionStatus.DESC;
        }
        String finalOrder = order;
        DirectionStatus finalDirection = direction;
        Page<Question> p= page(() -> {
            questionRelationMapper.listExerciseAdmin(structId, questionNo, paperId, place, difficult, startTime, endTime, finalOrder, finalDirection.key);
        }, page, pageSize);

        Collection ids = Transform.getIds(p, Question.class, "id");
        Transform.replace(p, select(ids), Question.class, "id");
        return p;
    }

    public Question add(Question question){
        int result = insert(questionMapper, question);
        question = one(questionMapper, question.getId());
        if(question == null){
            throw new SystemException("题目添加失败");
        }
        return question;
    }

    public Question edit(Question question){
        Question in = one(questionMapper, question.getId());
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = update(questionMapper, question);
        return question;
    }

    public boolean delete(Number id){
        Question in = one(questionMapper, id);
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = delete(questionMapper, id);
        return result > 0;
    }

    public Question selectOne(Number id){
        Question in = one(questionMapper, id);

        if(in == null){
            throw new ParameterException("题目不存在");
        }
        return in;
    }

    public Page<Question> select(int page, int pageSize){
        return select(questionMapper, page, pageSize);
    }

    public List<Question> select(Collection ids){
        return select(questionMapper, ids);
    }

}
