package com.qxgmat.service.inline;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.qxgmat.data.dao.SentenceQuestionMapper;
import com.qxgmat.data.dao.entity.Manager;
import com.qxgmat.data.dao.entity.Question;
import com.qxgmat.data.dao.entity.SentenceQuestion;
import com.qxgmat.data.relation.entity.SentenceQuestionRelation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class SentenceQuestionService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(SentenceQuestionService.class);

    @Resource
    private SentenceQuestionMapper sentenceQuestionMapper;

    public SentenceQuestion add(SentenceQuestion question){
        int result = insert(sentenceQuestionMapper, question);
        question = one(sentenceQuestionMapper, question.getId());
        if(question == null){
            throw new SystemException("题目添加失败");
        }
        return question;
    }

    public SentenceQuestion edit(SentenceQuestion question){
        SentenceQuestion in = one(sentenceQuestionMapper, question.getId());
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = update(sentenceQuestionMapper, question);
        return question;
    }

    public boolean delete(Number id){
        SentenceQuestion in = one(sentenceQuestionMapper, id);
        if(in == null){
            throw new ParameterException("题目不存在");
        }
        int result = delete(sentenceQuestionMapper, id);
        return result > 0;
    }

    public SentenceQuestion selectOne(Number id){
        SentenceQuestion in = one(sentenceQuestionMapper, id);

        if(in == null){
            throw new ParameterException("题目不存在");
        }
        return in;
    }

    public Page<SentenceQuestion> select(int page, int pageSize){
        return select(sentenceQuestionMapper, page, pageSize);
    }

    public Page<SentenceQuestion> select(int page, int pageSize, int chapter, int part){
        Example example = new Example(Manager.class);
        example.and(
                example.createCriteria()
                        .andEqualTo("chapter", chapter)
                        .andEqualTo("part", part)
        );
        return select(sentenceQuestionMapper, example, page, pageSize);
    }

    public List<SentenceQuestion> select(Collection ids){
        return select(sentenceQuestionMapper, ids);
    }

}
