package com.qxgmat.service.inline;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.constants.enums.user.AskTarget;
import com.qxgmat.data.constants.enums.user.MoneyRange;
import com.qxgmat.data.dao.UserAskMapper;
import com.qxgmat.data.dao.entity.UserAsk;
import com.qxgmat.data.relation.UserAskRelationMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class UserAskService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(UserAskService.class);

    @Resource
    private UserAskMapper userAskMapper;

    @Resource
    private UserAskRelationMapper userAskRelationMapper;

    public Page<UserAsk> listWithUser(int page, int size, Number category, Number userId, Number questionId, AskTarget target, Integer status, Integer showStatus, MoneyRange moneyRange, String order, DirectionStatus direction){
        String tk = target != null ? target.key : "";
        Integer max = moneyRange != null ? moneyRange.max == Integer.MAX_VALUE ? null : moneyRange.max : null;
        Integer min = moneyRange != null ? moneyRange.min : null;
        if(order.isEmpty()) order = "id";
        if (direction == null){
            direction = DirectionStatus.DESC;
        }
        String finalOrder = order;
        DirectionStatus finalDirection = direction;
        Page<UserAsk> p = page(
                ()-> userAskRelationMapper.listWithUser(category, userId, questionId, tk, status, showStatus, min, max, finalOrder, finalDirection.key)
        , page, size);

        Collection ids = Transform.getIds(p, UserAsk.class, "id");
        Transform.replace(p, select(ids), UserAsk.class, "id");
        return p;
    }

    public void adjust(Number questionId, Integer order, String direction, String flag){
        userAskRelationMapper.adjust(questionId, order, direction, flag);
    }

    @Transactional
    public void order(Number id, Integer order){
        UserAsk entity = selectOne(id);
        adjust(entity.getQuestionId(), entity.getOrder(), ">", "-=");
        adjust(entity.getQuestionId(), order, ">=", "+=");
        entity.setOrder(order);
    }

    public UserAsk add(UserAsk message){
        int result = insert(userAskMapper, message);
        message = one(userAskMapper, message.getId());
        if(message == null){
            throw new SystemException("提问添加失败");
        }
        return message;
    }

    public UserAsk edit(UserAsk message){
        UserAsk in = one(userAskMapper, message.getId());
        if(in == null){
            throw new ParameterException("提问不存在");
        }
        int result = update(userAskMapper, message);
        return message;
    }

    public boolean delete(Number id){
        UserAsk in = one(userAskMapper, id);
        if(in == null){
            throw new ParameterException("提问不存在");
        }
        int result = delete(userAskMapper, id);
        return result > 0;
    }

    public UserAsk selectOne(Number id){
        UserAsk in = one(userAskMapper, id);

        if(in == null){
            throw new ParameterException("提问不存在");
        }
        return in;
    }

    public Page<UserAsk> select(int page, int pageSize){
        return select(userAskMapper, page, pageSize);
    }

    public List<UserAsk> select(Collection ids){
        return select(userAskMapper, ids);
    }

}
